#ifndef __BRAINAPP__H__
#define __BRAINAPP__H__

#define QT_CLEAN_NAMESPACE 1

#include <iostream>
//#include <bicInventor.h>
#include <mniVertstatsFile.h>

#include "brainEnvironment.h"
#include "Rinterface.h"

// Inventor includes
#include <Inventor/SoOffscreenRenderer.h>
#include <Inventor/Qt/SoQt.h>
#include <Inventor/Qt/SoQtRenderArea.h>
#include <Inventor/Qt/viewers/SoQtExaminerViewer.h>

#include <Inventor/actions/SoBoxHighlightRenderAction.h>
#include <Inventor/actions/SoGetBoundingBoxAction.h>
#include <Inventor/actions/SoSearchAction.h>

#include <Inventor/events/SoKeyboardEvent.h>

#include <Inventor/nodes/SoBaseColor.h>
#include <Inventor/nodes/SoComplexity.h>
#include <Inventor/nodes/SoCone.h>
#include <Inventor/nodes/SoCoordinate3.h>
#include <Inventor/nodes/SoCube.h>
#include <Inventor/nodes/SoDirectionalLight.h>
#include <Inventor/nodes/SoDrawStyle.h>
#include <Inventor/nodes/SoEventCallback.h>
#include <Inventor/nodes/SoExtSelection.h>
#include <Inventor/nodes/SoFaceSet.h>
#include <Inventor/nodes/SoMaterial.h>
#include <Inventor/nodes/SoMaterialBinding.h>
#include <Inventor/nodes/SoPerspectiveCamera.h>
#include <Inventor/nodes/SoPolygonOffset.h>
#include <Inventor/nodes/SoSeparator.h>
#include <Inventor/nodes/SoSphere.h>
#include <Inventor/nodes/SoSwitch.h>
#include <Inventor/nodes/SoText2.h>
#include <Inventor/nodes/SoTexture2.h>
#include <Inventor/nodes/SoTexture2Transform.h>
#include <Inventor/nodes/SoTextureCoordinate2.h>
#include <Inventor/nodes/SoTextureCoordinateBinding.h>
#include <Inventor/nodes/SoTransform.h>
#include <Inventor/nodes/SoTranslation.h>

#include <Inventor/manips/SoClipPlaneManip.h>

#define QT_CLEAN_NAMESPACE 1
// QT includes
#include <qglobal.h>
#include <qapplication.h>
#include <qcursor.h>
#include <qhbox.h>
#include <qwidget.h>
#include <qobject.h>
#include <qlayout.h>
#include <qpopupmenu.h>
#include <qmenubar.h>
#include <qfiledialog.h>
#include <qcheckbox.h>
#include <qlineedit.h>
#include <qtextedit.h>
#include <qlistview.h>
#include <qcombobox.h>
#include <qcolordialog.h>
#include <qdropsite.h>
#include <qdir.h>
#include <qdragobject.h>

// brain-view includes
#include "pointInfoWindow.h"
#include "pointPicker.h"
#include "brainApp.h"
#include "vertStatsFileDialog.h"
#include "brainEnvironment.h"
#include "vertstatsForm.h"
#include "vertexInfo.h"
#include "screenCapture.h"
#include "options.h"
#include "RPlot.h"
#include "anatcon.h"
#include "brainWidget.h"


// class designed to handle all of the common variables
class brainApp : public QObject {
  Q_OBJECT
public:
  //! The constructor
  brainApp(int argc, char **argv, bool colourBar);
  //! loads a new image to use to paint the textures
  void setTextureImage(  QString imageFilename );
  //! loads a new obj file
  void setOBJFile( QString objFilename );
  //! loads a new vertstats file
  void setTextureFile( QString filename );
  //! sets the texture coordinate node
  void setTextureCoordinates( bool invert=false,
                              int intColumn=0,
                              QString stringColumn = "" );
  //! sets the texture range
  void setTextureRange( float min, float max );
  //! creates the scene graph - only called at startup
  void createSceneGraph();
  //! adds the SoSelection stuff to the scenegraph - only at startup
  void addExtselection();
  //! initialize the colour bar
  void initColourbar();
  //! sets the range widgets - only used at startup
  void updateRangeWidgets();

 public slots:
 //! load a new vertstats file
 void loadNewVertstatsFile( QString filename );
  //! the texture range has changed
  void textureRangeChanged();
  //! save the scene-graph to an inventor file.
  void saveSceneGraph();
  //! load a new file
  void newFile();
  //! show the plotting controls
  void plotWindowShow();
  //! show the anatomical connectivity controls
  void anatconWindowShow();
  //! show the vertex information window controls
  void vinfoWindowShow();
  //! load a new texture image (colour bar)
  void textureSelected(int listIndex);
  //! load a new column from the current vertstats file
  void vinfoHeaderSelected();
  //! populate the vertex info window with current vertstats headers
  void vinfoPopulate(float x=0, float y=0, float z=0, int=-1);
  //! set the background colour using a dialog
  void setBGColourDialog();
  //! quit the applicatoin
  void quit(); 
  //! plot the current point using the R interface
  void plotFromPickedPoint(float,float,float,int);
  //! trace anatomical connectivity at the current point using the R interface
  void traceFromPickedPoint(float,float,float,int);
  //! chosen when a new draw style has been picked
  void newDrawStyle(const QString&);

  //! paint a set of vertices
  void paintVertex(vector<int>);
  //! load a new painting vertstats file
  void loadPainting();
  //! save painting to vertstats file
  void savePainting();
  //! set all vertices to 0
  void resetPaint();
  //! dialog to set the paint label
  void setPaintLabel();
  //! toggle whether painting texture is enabled
  void togglePainting();
  //! flip direction of the clip plane
  void flipClipPane();
  //! toggle clip plane on or off
  void toggleClipPane();
  //! emits the signal which sends a list of painted vertices
  void sendPaintedVertices(int vertexNumber);
  void receivedNewFilename( QString );

  void setLocationSphere( int );
public:
  //private:
  //! The root to the object scene graph
  SoSeparator *rootObj;
  //! The revised root with extselection
  SoSeparator *paintRoot;
  //! The root to the texture info scene graph
  SoSeparator *textureInfo;
  //! The root to the (immature) minc window scene graph
  SoSeparator *rootMinc;
  //! The node that holds the geometry
  SoSeparator *geometry;
  //! The node that tells the geometry how to draw itself
  SoDrawStyle *geomDrawStyle;
  //! The extselection for painting
  SoExtSelection *selection;
  //! The texture image to use for colouring
  SoTexture2 *texture;
  //! The texture file representation
  mniVertstatsFile *vertstats;
  //! The texture coordinate in use
  SoTextureCoordinate2 *texCoord;
  //! The original texture coordinate - never scaled or transformed
  SoTextureCoordinate2 *originalTexCoord;
  SoMaterial *material;
  SoMaterialBinding *materialBinding;
  //! The texture complexity
  SoComplexity *textureComplexity;
  //! The texture transform used for scaling
  SoTexture2Transform *textureTrans;
  SoTextureCoordinateBinding *tBind;

  SoClipPlaneManip *clipper;
  SoSwitch *clipSwitch;

  // stuff for visual cue about vertex location
  SoSwitch *locationSphereSwitch;
  SoSphere *locationSphere;
  SoMaterial *locationSphereMaterial;
  SoTranslation *locationSphereTranslation;

  // holds the selected texture min and max
  float currentMin;
  float currentMax;

  // holds that textures absolute min and max
  float textureMin;
  float textureMax;

  //! A class that holds the environment variables from the initialisation
  brainEnvironment *environment;

  //! A class that opens a pipe to R
  Rinterface *Rpipe;

  //! Whether the scene graph has been instantiated.
  bool instantiated;

  //! Whether we have texture to play with
  bool haveTexture;

  SoEventCallback *mouseEventCB;
  SoPerspectiveCamera *camera;
  SoPerspectiveCamera *camera2;
  SoSwitch *textureSwitch;
  SoBaseColor *colour;
  SoCoordinate3 *geomCoordinates;

  // stuff for brain painting
  bool paintBrain;
  int paintNumber;
  SoSwitch *paintingSwitch;
  SoTexture2 *paintingTexture;
  SoTextureCoordinate2 *paintingTexCoord;
  SoTexture2Transform *paintingTexTrans;
  SoTextureCoordinateBinding *paintingTexBinding;
  SoComplexity *paintingTexComplexity;

  pointInfoWindow *infoWin;
  pointPicker *picker;
  vertstatsForm *vstatsWindow;
  vertexInfo *vinfoWindow;
  screenCapture *capturer;
  options *brainoptions;
  RPlot *plotter;
  anatcon *anatconTracer;
  brainWidget *mainWindow;
  QWidget *viewWindow;
  QWidget *textureWindow;
  QApplication *qapp;
  SoQtExaminerViewer *objViewer;
  SoQtRenderArea *textureViewer;
  SoText2 *lowColourmapValue;
  SoText2 *highColourmapValue;
  SoText2 *threeQColourmapValue;

  
  QLineEdit *topValue;
  QLineEdit *lowValue;
  QComboBox *textureList;

  signals:
  void newObjFile( QString newCortex );
  void paintedVertices( vector<int> vertices );
  

};


#endif // __BRAINAPP__H__





